@@ -1,13 +1,13 @@
 package Locale::CLDR::Locales::En::Any;
 
 # This file auto generated
-#	on Fri 10 Jul  9:02:08 pm GMT
+#	on Sat  6 Feb 10:36:44 am GMT
 
 use version;
 
-our $VERSION = version->declare('v0.27.2');
+our $VERSION = version->declare('v0.28.3');
 
-use v5.10;
+use v5.10.1;
 use mro 'c3';
 use if $^V ge v5.12.0, feature => 'unicode_strings';
 
