@@ -2,10 +2,10 @@ package Locale::CLDR::NumberFormatter;
 
 use version;
 
-our $VERSION = version->declare('v0.27.2');
+our $VERSION = version->declare('v0.28.3');
 
 
-use v5.10;
+use v5.10.1;
 use mro 'c3';
 use utf8;
 use if $^V ge v5.12.0, feature => 'unicode_strings';
@@ -27,6 +27,13 @@ sub format_number {
 	return $self->_format_number($number, $format, $currency, $for_cash);
 }
 
+sub format_currency {
+	my ($self, $number, $for_cash) = @_;
+	
+	my $format = $self->currency_format;
+	return $self->format_number($number, $format, undef(), $for_cash);
+}
+
 sub _format_number {
 	my ($self, $number, $format, $currency, $for_cash) = @_;
 	
@@ -70,7 +77,7 @@ sub add_currency_symbol {
 	my ($self, $format, $symbol) = @_;
 	
 	
-	$format =~ s/¤/'$symbol'/;
+	$format =~ s/¤/'$symbol'/g;
 	
 	return $format;
 }
@@ -112,23 +119,25 @@ sub parse_number_format {
 	$format = $self->add_currency_symbol($format, $currency)
 		if defined $currency;
 	
-	my ($positive, $negative) = $format =~ /^((?:(?:'[^']*')*+[^';]+)+) (?:;(.+))?$/x;
+	my ($positive, $negative) = $format =~ /^( (?: (?: ' [^']* ' )*+ | [^';]+ )+ ) (?: ; (.+) )? $/x;
+	
+	$negative //= "-$positive";
 	
 	my $type = 'positive';
 	foreach my $to_parse ( $positive, $negative ) {
-		last unless defined $to_parse;
 		my ($prefix, $suffix);
-		if (($prefix) = $to_parse =~ /^((?:[^0-9@#.,E+'*-] | (?:'[^']*')++)+)/x) {
-			$to_parse =~ s/^((?:[^0-9@#.,E+'*-] | (?:'[^']*')++)+)//x;
+		if (($prefix) = $to_parse =~ /^ ( (?: [^0-9@#.,E'*] | (?: ' [^']* ' )++ )+ ) /x) {
+			$to_parse =~ s/^ ( (?: [^0-9@#.,E'*] | (?: ' [^']* ' )++ )+ ) //x;
 		}
-		if( ($suffix) = $to_parse =~ /((?:[^0-9@#.,E+'-] | (?:'[^']*')++)+)$/x) {
-			$to_parse =~ s/((?:[^0-9@#.,E+'-] | (?:'[^']*')++)+)$//x;
+		if( ($suffix) = $to_parse =~ / ( (?: [^0-9@#.,E'] | (?: ' [^']* ' )++ )+ ) $ /x) {
+			$to_parse =~ s/( (?:[^0-9@#.,E'] | (?: ' [^']* ' )++ )+ ) $//x;
 		}
 		
-		# Fix escaped '
+		# Fix escaped ', - and +
 		foreach my $str ($prefix, $suffix) {
 			$str //= '';
-			$str =~ s/'((?:'')++ | [^']+)'/$1/gx;
+			$str =~ s/(?: ' (?: (?: '' )++ | [^']+ ) ' )*? \K ( [-+\\] ) /\\$1/gx;
+			$str =~ s/ ' ( (?: '' )++ | [^']++ ) ' /$1/gx;
 			$str =~ s/''/'/g;
 		}
 		
@@ -160,7 +169,7 @@ sub parse_number_format {
 		$multiplier = 100  if $prefix =~ tr/%/%/ || $suffix =~ tr/%/%/;
 		$multiplier = 1000 if $prefix =~ tr/‰/‰/ || $suffix =~ tr/‰/‰/;
 		
-		my $rounding = $to_parse =~ /([1-9][0-9]*(?:\.[0-9]+)?)/;
+		my $rounding = $to_parse =~ / ( [1-9] [0-9]* (?: \. [0-9]+ )? ) /x;
 		$rounding ||= 0;
 		
 		$rounding = $self->_get_currency_rounding($currency_data, $for_cash)
@@ -185,7 +194,7 @@ sub parse_number_format {
 		my $major_group;
 		my $minor_group;
 		if ($to_parse =~ tr/E/E/) {
-			($need_plus, $exponent) = $to_parse  =~ m/E(\+?)([0-9]+)/;
+			($need_plus, $exponent) = $to_parse  =~ m/ E ( \+? ) ( [0-9]+ ) /x;
 			$exponent_digits = length $exponent;
 		}
 		else {
@@ -217,7 +226,6 @@ sub parse_number_format {
 		$type = 'negative';
 	}
 	
-	$cache{$format}{negative} //= $cache{$format}{positive};
 	return $cache{$format};
 }
 
@@ -268,7 +276,7 @@ sub get_formatted_number {
 	
 	$symbols_type = $symbols{$symbols_type}{alias} if exists $symbols{$symbols_type}{alias};
 	
-	my $type = $number < 0 ? 'negative' : 'positive';
+	my $type = $number=~ s/^-// ? 'negative' : 'positive';
 	
 	$number *= $format->{$type}{multiplier};
 	
@@ -320,6 +328,15 @@ sub get_formatted_number {
 	foreach my $string ($prefix, $suffix) {
 		$string =~ s/%/$symbols{$symbols_type}{percentSign}/;
 		$string =~ s/‰/$symbols{$symbols_type}{perMille}/;
+		if ($type eq 'negative') {
+			$string =~ s/(?: \\ \\ )*+ \K \\ - /$symbols{$symbols_type}{minusSign}/x;
+			$string =~ s/(?: \\ \\)*+ \K \\ + /$symbols{$symbols_type}{minusSign}/x;
+		}
+		else {
+			$string =~ s/(?: \\ \\ )*+ \K \\ - //x;
+			$string =~ s/(?: \\ \\ )*+ \K \\ + /$symbols{$symbols_type}{plusSign}/x;
+		}
+		$string =~ s/ \\ \\ /\\/gx;
 	}
 	
 	$number = $prefix . $number . $suffix;
@@ -648,7 +665,7 @@ sub _get_algorithmic_number_format {
 		$previous = $key;
 	}
 }
-	
+
 no Moose::Role;
 
 1;
